@@ -17,13 +17,16 @@ limitations under the License.
 package model
 
 import (
+	"fmt"
 	"testing"
 	"time"
 
 	"github.com/stretchr/testify/assert"
 	apiv1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	vpa_types "k8s.io/autoscaler/vertical-pod-autoscaler/pkg/apis/autoscaling.k8s.io/v1beta2"
+	labels "k8s.io/apimachinery/pkg/labels"
+	vpa_types "k8s.io/autoscaler/vertical-pod-autoscaler/pkg/apis/autoscaling.k8s.io/v1"
+	"k8s.io/autoscaler/vertical-pod-autoscaler/pkg/utils/test"
 	"k8s.io/klog"
 )
 
@@ -82,6 +85,64 @@ func TestClusterGCAggregateContainerStateDeletesOld(t *testing.T) {
 	assert.Empty(t, vpa.aggregateContainerStates)
 }
 
+func TestClusterGCAggregateContainerStateDeletesOldEmpty(t *testing.T) {
+	// Create a pod with a single container.
+	cluster := NewClusterState()
+	vpa := addTestVpa(cluster)
+	addTestPod(cluster)
+
+	assert.NoError(t, cluster.AddOrUpdateContainer(testContainerID, testRequest))
+	// No usage samples added.
+
+	assert.NotEmpty(t, cluster.aggregateStateMap)
+	assert.NotEmpty(t, vpa.aggregateContainerStates)
+
+	assert.Len(t, cluster.aggregateStateMap, 1)
+	var creationTime time.Time
+	for _, aggregateState := range cluster.aggregateStateMap {
+		creationTime = aggregateState.CreationTime
+	}
+
+	// Verify empty aggregate states are not removed right away.
+	cluster.GarbageCollectAggregateCollectionStates(creationTime.Add(1 * time.Minute)) // AggegateContainerState should be deleted from both cluster and vpa
+	assert.NotEmpty(t, cluster.aggregateStateMap)
+	assert.NotEmpty(t, vpa.aggregateContainerStates)
+
+	// AggegateContainerState are valid for 8 days since creation
+	cluster.GarbageCollectAggregateCollectionStates(creationTime.Add(9 * 24 * time.Hour))
+
+	// AggegateContainerState should be deleted from both cluster and vpa
+	assert.Empty(t, cluster.aggregateStateMap)
+	assert.Empty(t, vpa.aggregateContainerStates)
+}
+
+func TestClusterGCAggregateContainerStateDeletesEmptyInactive(t *testing.T) {
+	// Create a pod with a single container.
+	cluster := NewClusterState()
+	vpa := addTestVpa(cluster)
+	pod := addTestPod(cluster)
+
+	assert.NoError(t, cluster.AddOrUpdateContainer(testContainerID, testRequest))
+	// No usage samples added.
+
+	assert.NotEmpty(t, cluster.aggregateStateMap)
+	assert.NotEmpty(t, vpa.aggregateContainerStates)
+
+	cluster.GarbageCollectAggregateCollectionStates(testTimestamp)
+
+	// AggegateContainerState should not be deleted as the pod is still active.
+	assert.NotEmpty(t, cluster.aggregateStateMap)
+	assert.NotEmpty(t, vpa.aggregateContainerStates)
+
+	cluster.Pods[pod.ID].Phase = apiv1.PodSucceeded
+	cluster.GarbageCollectAggregateCollectionStates(testTimestamp)
+
+	// AggegateContainerState should be empty as the pod is no longer active and
+	// there are no usage samples.
+	assert.Empty(t, cluster.aggregateStateMap)
+	assert.Empty(t, vpa.aggregateContainerStates)
+}
+
 func TestClusterGCAggregateContainerStateLeavesValid(t *testing.T) {
 	// Create a pod with a single container.
 	cluster := NewClusterState()
@@ -328,3 +389,200 @@ func TestEmptySelector(t *testing.T) {
 	assert.Contains(t, vpa.aggregateContainerStates, cluster.aggregateStateKeyForContainerID(containerID1))
 	assert.Contains(t, vpa.aggregateContainerStates, cluster.aggregateStateKeyForContainerID(containerID2))
 }
+
+func TestRecordRecommendation(t *testing.T) {
+	cases := []struct {
+		name               string
+		recommendation     *vpa_types.RecommendedPodResources
+		lastLogged         time.Time
+		now                time.Time
+		expectedEmpty      bool
+		expectedLastLogged time.Time
+		expectedError      error
+	}{
+		{
+			name:           "VPA has recommendation",
+			recommendation: test.Recommendation().WithContainer("test").WithTarget("100m", "200G").Get(),
+			now:            testTimestamp,
+			expectedEmpty:  false,
+			expectedError:  nil,
+		}, {
+			name:           "VPA recommendation appears",
+			recommendation: test.Recommendation().WithContainer("test").WithTarget("100m", "200G").Get(),
+			lastLogged:     testTimestamp.Add(-10 * time.Minute),
+			now:            testTimestamp,
+			expectedEmpty:  false,
+			expectedError:  nil,
+		}, {
+			name:               "VPA recommendation missing",
+			recommendation:     &vpa_types.RecommendedPodResources{},
+			lastLogged:         testTimestamp.Add(-10 * time.Minute),
+			now:                testTimestamp,
+			expectedEmpty:      true,
+			expectedLastLogged: testTimestamp.Add(-10 * time.Minute),
+			expectedError:      nil,
+		}, {
+			name:               "VPA recommendation missing and needs logging",
+			recommendation:     &vpa_types.RecommendedPodResources{},
+			lastLogged:         testTimestamp.Add(-40 * time.Minute),
+			now:                testTimestamp,
+			expectedEmpty:      true,
+			expectedLastLogged: testTimestamp,
+			expectedError:      fmt.Errorf("VPA namespace-1/vpa-1 is missing recommendation for more than %v", RecommendationMissingMaxDuration),
+		}, {
+			name:               "VPA recommendation disappears",
+			recommendation:     &vpa_types.RecommendedPodResources{},
+			now:                testTimestamp,
+			expectedEmpty:      true,
+			expectedLastLogged: testTimestamp,
+			expectedError:      nil,
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			cluster := NewClusterState()
+			vpa := addVpa(cluster, testVpaID, testSelectorStr)
+			cluster.Vpas[testVpaID].Recommendation = tc.recommendation
+			if !tc.lastLogged.IsZero() {
+				cluster.EmptyVPAs[testVpaID] = tc.lastLogged
+			}
+
+			err := cluster.RecordRecommendation(vpa, tc.now)
+			if tc.expectedError != nil {
+				assert.Equal(t, tc.expectedError, err)
+			} else {
+				assert.NoError(t, err)
+				if tc.expectedEmpty {
+					assert.Contains(t, cluster.EmptyVPAs, testVpaID)
+					assert.Equal(t, cluster.EmptyVPAs[testVpaID], tc.expectedLastLogged)
+				} else {
+					assert.NotContains(t, cluster.EmptyVPAs, testVpaID)
+				}
+			}
+		})
+	}
+}
+
+type podDesc struct {
+	id     PodID
+	labels labels.Set
+	phase  apiv1.PodPhase
+}
+
+func TestGetActiveMatchingPods(t *testing.T) {
+	cases := []struct {
+		name         string
+		vpaSelector  string
+		pods         []podDesc
+		expectedPods []PodID
+	}{
+		{
+			name:         "No pods",
+			vpaSelector:  testSelectorStr,
+			pods:         []podDesc{},
+			expectedPods: []PodID{},
+		}, {
+			name:        "Matching pod",
+			vpaSelector: testSelectorStr,
+			pods: []podDesc{
+				{
+					id:     testPodID,
+					labels: testLabels,
+					phase:  apiv1.PodRunning,
+				},
+			},
+			expectedPods: []PodID{testPodID},
+		}, {
+			name:        "Matching pod is inactive",
+			vpaSelector: testSelectorStr,
+			pods: []podDesc{
+				{
+					id:     testPodID,
+					labels: testLabels,
+					phase:  apiv1.PodFailed,
+				},
+			},
+			expectedPods: []PodID{testPodID},
+		}, {
+			name:        "No matching pods",
+			vpaSelector: testSelectorStr,
+			pods: []podDesc{
+				{
+					id:     testPodID,
+					labels: emptyLabels,
+					phase:  apiv1.PodRunning,
+				}, {
+					id:     PodID{Namespace: "different-than-vpa", PodName: "pod-1"},
+					labels: testLabels,
+					phase:  apiv1.PodRunning,
+				},
+			},
+			expectedPods: []PodID{},
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			cluster := NewClusterState()
+			vpa := addVpa(cluster, testVpaID, tc.vpaSelector)
+			for _, pod := range tc.pods {
+				cluster.AddOrUpdatePod(pod.id, pod.labels, pod.phase)
+			}
+			pods := cluster.GetMatchingPods(vpa)
+			assert.ElementsMatch(t, tc.expectedPods, pods)
+		})
+	}
+}
+
+func TestVPAWithMatchingPods(t *testing.T) {
+	cases := []struct {
+		name          string
+		vpaSelector   string
+		pods          []podDesc
+		expectedMatch bool
+	}{
+		{
+			name:          "No pods",
+			vpaSelector:   testSelectorStr,
+			pods:          []podDesc{},
+			expectedMatch: false,
+		},
+		{
+			name:        "VPA with matching pod",
+			vpaSelector: testSelectorStr,
+			pods: []podDesc{
+				{
+					testPodID,
+					testLabels,
+					apiv1.PodRunning,
+				},
+			},
+			expectedMatch: true,
+		},
+		{
+			name:        "No matching pod",
+			vpaSelector: testSelectorStr,
+			pods: []podDesc{
+				{
+					testPodID,
+					emptyLabels,
+					apiv1.PodRunning,
+				},
+			},
+			expectedMatch: false,
+		},
+	}
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			cluster := NewClusterState()
+			vpa := addVpa(cluster, testVpaID, tc.vpaSelector)
+			for _, podDesc := range tc.pods {
+				cluster.AddOrUpdatePod(podDesc.id, podDesc.labels, podDesc.phase)
+				containerID := ContainerID{testPodID, "foo"}
+				assert.NoError(t, cluster.AddOrUpdateContainer(containerID, testRequest))
+			}
+			assert.Equal(t, tc.expectedMatch, cluster.VpasWithMatchingPods[vpa.ID])
+		})
+	}
+}
